//+------------------------------------------------------------------+
//|                                                ZigzagHistory.mq4 |
//|                 Copyright  2005-2007, MetaQuotes Software Corp. |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, MetaQuotes Software Corp."

#property indicator_chart_window
#property indicator_buffers 6
#property indicator_color1 Red
#property indicator_color4 Green
#property indicator_color5 Green
#property indicator_color6 DimGray
//---- indicator parameters
extern ENUM_TIMEFRAMES TimeFrame = PERIOD_CURRENT;  // Time frame to use
extern int ExtDepth=10;
extern int ExtDeviation=5;
extern int ExtBackstep=3;
//---- indicator buffers
double ZigzagBuffer[];
double HighMapBuffer[];
double LowMapBuffer[];
double Hi[];
double Lo[];
double Mid[];
int level=3; // recounting's depth 
bool downloadhistory=false;
string indicatorFileName;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   IndicatorBuffers(6);
//---- drawing settings
//---- indicator buffers mapping
   SetIndexBuffer(0,ZigzagBuffer); SetIndexStyle(0,DRAW_SECTION);
   SetIndexBuffer(1,HighMapBuffer); SetIndexStyle(1,DRAW_NONE);
   SetIndexBuffer(2,LowMapBuffer); SetIndexStyle(2,DRAW_NONE);
   SetIndexBuffer(3,Hi); SetIndexStyle(3,DRAW_LINE);
   SetIndexBuffer(4,Lo); SetIndexStyle(4,DRAW_LINE);
   SetIndexBuffer(5,Mid); SetIndexStyle(5,DRAW_LINE);
   SetIndexEmptyValue(0,0.0);
            indicatorFileName = WindowExpertName();
            TimeFrame         = MathMax(TimeFrame,_Period);

//---- indicator short name
   IndicatorShortName("ZigZag("+ExtDepth+","+ExtDeviation+","+ExtBackstep+")");
//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
  {
   int i, counted_bars = IndicatorCounted();
      if (TimeFrame!=_Period)
      {
         for(int shift=Bars-1; shift>=0; shift--)
         {
            int y = iBarShift(NULL,TimeFrame,Time[shift]);
            int x = iBarShift(NULL,TimeFrame,Time[(int)MathMin(shift+1,Bars-1)]);
            
            if (x!=y) 
                  ZigzagBuffer[shift] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,ExtDepth,ExtDeviation,ExtBackstep,0,y);
            else  ZigzagBuffer[shift] = EMPTY_VALUE;
                  Hi [shift] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,ExtDepth,ExtDeviation,ExtBackstep,3,y);
                  Lo [shift] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,ExtDepth,ExtDeviation,ExtBackstep,4,y);
                  Mid[shift] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,ExtDepth,ExtDeviation,ExtBackstep,5,y);
         }   
         return(0);
      }         
   int limit,counterZ,whatlookfor;
   int back,lasthighpos,lastlowpos;
   double val,res;
   double curlow,curhigh,lasthigh,lastlow;

   if (counted_bars==0 && downloadhistory) // history was downloaded
     {
      ArrayInitialize(ZigzagBuffer,0.0);
      ArrayInitialize(HighMapBuffer,0.0); 
      ArrayInitialize(LowMapBuffer,0.0);
      ArrayInitialize(Hi,0.0);
      ArrayInitialize(Lo,0.0);
      ArrayInitialize(Mid,0.0);
     }
   if (counted_bars==0) 
     {
      limit=Bars-ExtDepth;
      downloadhistory=true;
     }
   if (counted_bars>0) 
     {
      while (counterZ<level && i<100)
        {
         res=ZigzagBuffer[i];
         if (res!=0) counterZ++;
         i++;
        }
      i--;
      limit=i;
      if (LowMapBuffer[i]!=0) 
        {
         curlow=LowMapBuffer[i];
         whatlookfor=1;
        }
      else
        {
         curhigh=HighMapBuffer[i];
         whatlookfor=-1;
        }
      for (i=limit-1;i>=0;i--)  
        {
         ZigzagBuffer[i]=0.0;  
         LowMapBuffer[i]=0.0;
         HighMapBuffer[i]=0.0;
        }
     }
      
   for(shift=limit; shift>=0; shift--)
     {
      val=Low[iLowest(NULL,0,MODE_LOW,ExtDepth,shift)];
      if(val==lastlow) val=0.0;
      else 
        { 
         lastlow=val; 
         if((Low[shift]-val)>(ExtDeviation*Point)) val=0.0;
         else
           {
            for(back=1; back<=ExtBackstep; back++)
              {
               res=LowMapBuffer[shift+back];
               if((res!=0)&&(res>val)) LowMapBuffer[shift+back]=0.0; 
              }
           }
        } 
      if (Low[shift]==val) {	LowMapBuffer[shift]=val; Lo[shift]=val; }	
      else {	LowMapBuffer[shift]=0.0; Lo[shift] = Lo[shift+1];}
      //--- high
      val=High[iHighest(NULL,0,MODE_HIGH,ExtDepth,shift)];
      if(val==lasthigh) val=0.0;
      else 
        {
         lasthigh=val;
         if((val-High[shift])>(ExtDeviation*Point)) val=0.0;
         else
           {
            for(back=1; back<=ExtBackstep; back++)
              {
               res=HighMapBuffer[shift+back];
               if((res!=0)&&(res<val)) HighMapBuffer[shift+back]=0.0; 
              } 
           }
        }
      if (High[shift]==val) {	HighMapBuffer[shift]=val; Hi[shift]=val; }
      else {	HighMapBuffer[shift]=0.0; Hi[shift]=Hi[shift+1]; }
     }

	for (shift=limit;shift>=0;shift--)  Mid[shift] = (Hi[shift]+Lo[shift])/2;

   // final cutting 
   if (whatlookfor==0)
     {
      lastlow=0;
      lasthigh=0;  
     }
   else
     {
      lastlow=curlow;
      lasthigh=curhigh;
     }
   for (shift=limit;shift>=0;shift--)
     {
      res=0.0;
      switch(whatlookfor)
        {
         case 0: // look for peak or lawn 
            if (lastlow==0 && lasthigh==0)
              {
               if (HighMapBuffer[shift]!=0)
                 {
                  lasthigh=High[shift];
                  lasthighpos=shift;
                  whatlookfor=-1;
                  ZigzagBuffer[shift]=lasthigh;
                  res=1;
                 }
               if (LowMapBuffer[shift]!=0)
                 {
                  lastlow=Low[shift];
                  lastlowpos=shift;
                  whatlookfor=1;
                  ZigzagBuffer[shift]=lastlow;
                  res=1;
                 }
              }
             break;  
         case 1: // look for peak
            if (LowMapBuffer[shift]!=0.0 && LowMapBuffer[shift]<lastlow && HighMapBuffer[shift]==0.0)
              {
               ZigzagBuffer[lastlowpos]=0.0;
               lastlowpos=shift;
               lastlow=LowMapBuffer[shift];
               ZigzagBuffer[shift]=lastlow;
               res=1;
              }
            if (HighMapBuffer[shift]!=0.0 && LowMapBuffer[shift]==0.0)
              {
               lasthigh=HighMapBuffer[shift];
               lasthighpos=shift;
               ZigzagBuffer[shift]=lasthigh;
               whatlookfor=-1;
               res=1;
              }   
            break;               
         case -1: // look for lawn
            if (HighMapBuffer[shift]!=0.0 && HighMapBuffer[shift]>lasthigh && LowMapBuffer[shift]==0.0)
              {
               ZigzagBuffer[lasthighpos]=0.0;
               lasthighpos=shift;
               lasthigh=HighMapBuffer[shift];
               ZigzagBuffer[shift]=lasthigh;
              }
            if (LowMapBuffer[shift]!=0.0 && HighMapBuffer[shift]==0.0)
              {
               lastlow=LowMapBuffer[shift];
               lastlowpos=shift;
               ZigzagBuffer[shift]=lastlow;
               whatlookfor=1;
              }   
            break;               
         default: return(0); 
        }
     }

   return(0);
  }
//+------------------------------------------------------------------+